'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _AppStateWithEffects = require('./AppStateWithEffects');

var _AppStateWithEffects2 = _interopRequireDefault(_AppStateWithEffects);

var _utils = require('./utils');

/**
 * Redux standard middleware. The middleware is responsible for pulling the AppStateWithEffects
 * off the Store.getState result. If the state is instance of AppStateWithEffects all the
 * effects (Functions) gets executed and dispatch is provided as the argument.
 *
 * @returns {Function} Redux standard middleware
 */

exports['default'] = function (store) {
  return function (next) {
    return function (action) {
      var result = next(action);

      var storeState = store.getState();
      if (storeState instanceof _AppStateWithEffects2['default']) {
        var effects = storeState.getEffects();

        (0, _utils.invariant)(effects.every(_utils.isFunction), 'It\'s allowed to yield only functions (side effect)');

        // Side effect is just an yielded Function which receives dispatch as the first argument.
        effects.forEach(function (effect) {
          return effect(store.dispatch);
        });
      }

      return result;
    };
  };
};

module.exports = exports['default'];